﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace GLI_BitmapEditor
{
    public partial class Form1 : Form
    {
        private bool[,] bitmapData;
        private int width, height;

        private Bitmap bitmap;
        private Graphics graphics;

        public Form1()
        {
            InitializeComponent();

            ClientSize = new Size(769, 385 + menuStrip1.Height + statusStrip1.Height);

            NewBitmap(128, 64);
        }

        private void NewBitmap(int width, int height)
        {
            this.width = width;
            this.height = height;

            bitmapData = new bool[height, width];

            bitmap = new Bitmap(6 * width + 1, 6 * height + 1);
            graphics = Graphics.FromImage(bitmap);

            graphics.Clear(Color.DarkGray);

            for (int y = 0; y < height; y++)
            {
                for (int x = 0; x < width; x++)
                    graphics.FillRectangle(Brushes.White, 1 + 6 * x, 1 + 6 * y, 5, 5);
            }

            Invalidate();
        }

        private void SetPixel(int x, int y, bool color)
        {
            bitmapData[y, x] = color;

            if (color)
                graphics.FillRectangle(Brushes.Black, 1 + 6 * x, 1 + 6 * y, 5, 5);
            else
                graphics.FillRectangle(Brushes.White, 1 + 6 * x, 1 + 6 * y, 5, 5);

            Invalidate();
        }

        private void Invert()
        {
            graphics.Clear(Color.DarkGray);

            for (int y = 0; y < height; y++)
            {
                for (int x = 0; x < width; x++)
                {
                    bitmapData[y, x] = !bitmapData[y, x];

                    if (bitmapData[y, x])
                        graphics.FillRectangle(Brushes.Black, 1 + 6 * x, 1 + 6 * y, 5, 5);
                    else
                        graphics.FillRectangle(Brushes.White, 1 + 6 * x, 1 + 6 * y, 5, 5);
                }
            }

            Invalidate();
        }

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.DrawImage(bitmap, 0, menuStrip1.Height);
        }

        private void Form1_MouseMove(object sender, MouseEventArgs e)
        {
            int x = (e.X - 1) / 6;
            int y = (e.Y - 1 - menuStrip1.Height) / 6;

            toolStripStatusLabel1.Text = "X: " + x + "; Y: " + y;

            if (x < 0 || x >= width || y < 0 || y >= height)
                return;

            if (e.Button == System.Windows.Forms.MouseButtons.Left)
                SetPixel(x, y, true);

            if (e.Button == System.Windows.Forms.MouseButtons.Right)
                SetPixel(x, y, false);
        }

        private void newToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Form2 newBitmapForm = new Form2();

            newBitmapForm.BitmapWidth = width;
            newBitmapForm.BitmapHeight = height;

            if (newBitmapForm.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                NewBitmap(newBitmapForm.BitmapWidth, newBitmapForm.BitmapHeight);
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Form3 saveBitmapForm = new Form3();

            saveBitmapForm.Output =
                "const unsigned char PROGMEM g_pBitmapData[] =\r\n" +
                "{\r\n";

            byte data = 0;
            int bit = 0;
            int bytes = 0;

            for (int y = 0; y < height; y++)
            {
                for (int x = 0; x < width; x++)
                {
                    if (bitmapData[y, x])
                        data |= (byte)(0x80 >> bit);

                    bit++;

                    if (bit == 8)
                    {
                        if (bytes == 0)
                            saveBitmapForm.Output += "    ";

                        if (y == height - 1 && x == width - 1)
                            saveBitmapForm.Output += "0x" + data.ToString("x2") + "\r\n";
                        else
                            saveBitmapForm.Output += "0x" + data.ToString("x2") + ", ";

                        bytes++;

                        if (bytes == 12)
                        {
                            if (y != height - 1 || x != width - 1)
                                saveBitmapForm.Output += "\r\n";

                            bytes = 0;
                        }

                        data = 0;
                        bit = 0;

                    }
                }
            }

            if (bit != 0)
            {
                if (bytes == 0)
                    saveBitmapForm.Output += "    ";

                saveBitmapForm.Output += "0x" + data.ToString("x2") + "\r\n";
            }

            saveBitmapForm.Output +=
                "};\r\n" +
                "\r\n" +
                "\r\n" +
                "const GLI_BITMAP g_Bitmap =\r\n" +
                "{\r\n" +
                "    " + width + ",\r\n" +
                "    " + height + ",\r\n" +
                "    g_pBitmapData\r\n" +
                "};";

            saveBitmapForm.ShowDialog();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void clearToolStripMenuItem_Click(object sender, EventArgs e)
        {
            NewBitmap(width, height);
        }

        private void invertToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Invert();
        }
    }
}
