﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace GLI_FontGenerator
{
    public partial class Form1 : Form
    {
        private Font font;

        private int charWidth;
        private int charHeight;

        public Form1()
        {
            InitializeComponent();

            comboBox1.Items.Add("Space");
            comboBox2.Items.Add("Space");

            for (char c = (char)33; c < 127; c++)
            {
                comboBox1.Items.Add(c);
                comboBox2.Items.Add(c);
            }

            comboBox1.SelectedIndex = 0;
            comboBox2.SelectedIndex = comboBox2.Items.Count - 1;

            SetFont();
        }

        private void SetFont()
        {
            font = fontDialog1.Font;

            charWidth = (int)font.Size;
            charHeight = font.Height;

            label6.Text = charWidth.ToString();
            label7.Text = charHeight.ToString();
        }

        private Bitmap GetFontBitmap()
        {
            char c = (char)0;

            Bitmap bitmap = new Bitmap(charWidth * 19 + 20, charHeight * 5 + 6);
            Graphics graphics = Graphics.FromImage(bitmap);

            graphics.TextRenderingHint = System.Drawing.Text.TextRenderingHint.SingleBitPerPixelGridFit;

            graphics.Clear(Color.DarkGray);

            for (int y = 0; y < 5; y++)
            {
                for (int x = 0; x < 19; x++)
                {
                    Brush brush;
                    if (comboBox1.SelectedIndex <= c && c <= comboBox2.SelectedIndex)
                        brush = Brushes.White;
                    else
                        brush = Brushes.LightGray;

                    graphics.FillRectangle(
                        brush,
                        1 + (charWidth + 1) * x,
                        1 + (charHeight + 1) * y,
                        charWidth, charHeight);

                    graphics.DrawString(
                        ((char)(c + 32)).ToString(),
                        font, Brushes.Black,
                        1 + (charWidth + 1) * x,
                        1 + (charHeight + 1) * y);

                    c++;
                }
            }

            return bitmap;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (fontDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                SetFont();
        }

        private void comboBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox2.SelectedIndex < comboBox1.SelectedIndex)
                comboBox2.SelectedIndex = comboBox1.SelectedIndex;
        }

        private void comboBox2_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboBox1.SelectedIndex > comboBox2.SelectedIndex)
                comboBox1.SelectedIndex = comboBox2.SelectedIndex;
        }

        private void button2_Click(object sender, EventArgs e)
        {
            new Form2(GetFontBitmap()).ShowDialog();
        }

        private void button3_Click(object sender, EventArgs e)
        {
            Form3 saveFontForm = new Form3();

            Bitmap bitmap = GetFontBitmap();

            saveFontForm.Output =
                "const unsigned char PROGMEM g_pFontData[] =\r\n" +
                "{\r\n";

            for (int i = 0; i < 5; i++)
            {
                for (int j = 0; j < 19; j++)
                {
                    int charIndex = i * 19 + j;

                    if (charIndex < comboBox1.SelectedIndex ||
                        charIndex > comboBox2.SelectedIndex)
                        continue;

                    byte data = 0;
                    int bit = 0;

                    saveFontForm.Output += "    ";

                    for (int y = 0; y < charHeight; y++)
                    {
                        for (int x = 0; x < charWidth; x++)
                        {
                            Color color = bitmap.GetPixel(1 + (charWidth + 1) * j + x, 1 + (charHeight + 1) * i + y);

                            if (color.R == 0 && color.G == 0 && color.B == 0)
                                data |= (byte)(0x80 >> bit);

                            bit++;

                            if (bit == 8)
                            {
                                if (y == charHeight - 1 && x == charWidth - 1)
                                {
                                    if (charIndex == comboBox2.SelectedIndex)
                                        saveFontForm.Output += "0x" + data.ToString("x2") + "\r\n";
                                    else
                                        saveFontForm.Output += "0x" + data.ToString("x2") + ",\r\n";
                                }
                                else
                                {
                                    saveFontForm.Output += "0x" + data.ToString("x2") + ", ";
                                }

                                data = 0;
                                bit = 0;
                            }
                        }
                    }

                    if (bit != 0)
                    {
                        if (charIndex == comboBox2.SelectedIndex)
                            saveFontForm.Output += "0x" + data.ToString("x2") + "\r\n";
                        else
                            saveFontForm.Output += "0x" + data.ToString("x2") + ",\r\n";
                    }
                }
            }

            int charSize = charWidth * charHeight;
            charSize = charSize / 8 + ((charSize % 8 != 0) ? 1 : 0);

            saveFontForm.Output +=
                "};\r\n" +
                "\r\n" +
                "\r\n" +
                "const GLI_FONT g_Font =\r\n" +
                "{\r\n" +
                "    " + charWidth + ",\r\n" +
                "    " + charHeight + ",\r\n" +
                "    " + charSize + ",\r\n" +
                "    " + (32 + comboBox1.SelectedIndex) + ",\r\n" +
                "    g_pFontData\r\n" +
                "};";

            saveFontForm.ShowDialog();
        }
    }
}
